searchData={"items":[{"type":"module","title":"LemonCrud","doc":"The `LemonCrud` module generates common [CRUD](https://pl.wikipedia.org/wiki/CRUD) (Create, Read, Update, Delete) functions for a context, similar to what `mix phx gen context` task generates.","ref":"LemonCrud.html"},{"type":"module","title":"Options - LemonCrud","doc":"- `:repo` - The Ecto repository module used for database operations (required)\n- `:schema` - The Ecto schema module representing the resource that these CRUD operations will be generated for (required)\n- `:exclude` - A list of atoms representing the functions to be excluded from generation (optional)\n- `:plural_resource_name` - A custom plural version of the resource name to be used in function names (optional). If not provided, singular version with 's' ending will be used to generate list function","ref":"LemonCrud.html#module-options"},{"type":"module","title":"Usage - LemonCrud","doc":"```elixir\ndefmodule MyApp.Accounts do\n  use LemonCrud,\n    repo: MyApp.Repo,\n    schema: MyApp.Accounts.User,\n    exclude: [:delete],\n    plural_resource_name: \"users\"\nend\n```\n\nThis sample usage will generate all CRUD functions for `MyApp.Accounts.User` resource, excluding `delete_user/1`.","ref":"LemonCrud.html#module-usage"},{"type":"module","title":"Generated Functions - LemonCrud","doc":"The following functions are generated by default. Any of them can be excluded by adding their correspoding atom to the `:exclude` option.\n\n- `list_{plural resource name}` - Lists all resources in the schema, optionally filtered by a query or condition list, or preloaded with associations.\n- `get_{resource name}` - Retrieves a resource by its ID, optionally preloaded with associations. Returns `nil` if not found.\n- `get_{resource name}!` - Retrieves a resource by its ID, optionally preloaded with associations. Raises an error if not found.\n- `get_{resource name}_by` - Retrieves a resource by a query or condition list, optionally preloaded with associations. Returns `nil` if not found.\n- `get_{resource name}_by!` - Retrieves a resource by a query or condition list, optionally preloaded with associations. Raises an error if not found.\n- `create_{resource name}` - Creates a new resource with the provided attributes. Returns an `:ok` tuple with the resource or an `:error` tuple with changeset.\n- `create_{resource name}!` - Creates a new resource with the provided attributes. Raises an error if creation fails.\n- `update_{resource name}` - Updates an existing resource with the provided attributes. Returns an `:ok` tuple with the resource or an `:error` tuple with changeset.\n- `update_{resource name}!` - Updates an existing resource with the provided attributes. Raises an error if update fails.\n- `delete_{resource name}` - Deletes an existing resource. Returns an `:ok` tuple with the resource or an `:error` tuple with changeset.\n- `delete_{resource name}!` - Deletes an existing resource. Raises an error if delete fails.\n- `change_{resource name}` - Returns changeset for given resource.\n\n{resource name} and {plural resource name} will be replaced by the singular and plural forms of the resource name.","ref":"LemonCrud.html#module-generated-functions"},{"type":"module","title":"LemonCrud.QueryBuilder","doc":"Builds queries based on conditions defined as keyword lists.","ref":"LemonCrud.QueryBuilder.html"},{"type":"module","title":"Example - LemonCrud.QueryBuilder","doc":"```elixir\nquery = LemonCrud.QueryBuilder.build_query(MyApp.Inventory.Item, [\n  part_number: \"1234567890\",\n  category: \"electronics\",\n  manufacturer: [name: \"Acme\"]\n])\n\n# This will generate the following query:\n#\n# from i in MyApp.Inventory.Item,\n#   join: m in assoc(i, :manufacturer),\n#   where: i.category == \"electronics\" and m.name == \"Acme\" and i.part_number == \"1234567890\"\n```","ref":"LemonCrud.QueryBuilder.html#module-example"},{"type":"function","title":"LemonCrud.QueryBuilder.build_query/3","doc":"Builds a query based on the given schema and conditions.\n\nAutomatically joins associations based on the conditions.","ref":"LemonCrud.QueryBuilder.html#build_query/3"},{"type":"function","title":"Example - LemonCrud.QueryBuilder.build_query/3","doc":"```elixir\nquery = LemonCrud.QueryBuilder.build_query(MyApp.Inventory.Item, [\n  part_number: \"1234567890\",\n  category: \"electronics\",\n  manufacturer: [name: \"Acme\"]\n])\n\n# This will generate the following query:\n#\n# from i in MyApp.Inventory.Item,\n#   join: m in assoc(i, :manufacturer),\n#   where: i.category == \"electronics\" and m.name == \"Acme\" and i.part_number == \"1234567890\"\n```","ref":"LemonCrud.QueryBuilder.html#build_query/3-example"},{"type":"extras","title":"LemonCrud","doc":"# LemonCrud\n\n[![Hex.pm](https://img.shields.io/hexpm/v/lemon_crud.svg)](https://hex.pm/packages/lemon_crud)\n[![CI](https://github.com/curiosum-dev/lemon_crud/actions/workflows/ci.yml/badge.svg)](https://github.com/curiosum-dev/lemon_crud/actions/workflows/ci.yml)\n[![Coverage Status](https://codecov.io/gh/curiosum-dev/lemon_crud/branch/main/graph/badge.svg)](https://codecov.io/gh/curiosum-dev/lemon_crud)\n[![License](https://img.shields.io/hexpm/l/lemon_crud.svg)](https://github.com/curiosum-dev/lemon_crud/blob/main/LICENSE)\n\nLemonCrud is a small library creating uniform yet flexible CRUD functions for your Phoenix contexts to reduce generated boilerplate.\n\nPaired with [Contexted](https://github.com/curiosum-dev/contexted), it adds a productivity benefit on top of Contexted's help in organizing Phoenix contexts in a nice and tidy way.","ref":"readme.html"},{"type":"extras","title":"Installation - LemonCrud","doc":"The package can be installed\nby adding `lemon_crud` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:lemon_crud, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - LemonCrud","doc":"In most web apps CRUD operations are very common. Most of these, have the same pattern. Most of the time, they are used with preloading associated resources as well as filtering based on conditions such as search, pagination, etc.\n\nWhy not autogenerate them?\n\nHere is how you can generate common CRUD operations for `App.Account.Users`:\n\n```elixir\ndefmodule App.Account.Users do\n  use LemonCrud,\n    repo: App.Repo,\n    schema: App.Accounts.User\nend\n```\n\nThis will generate the following functions:\n\n```elixir\niex> App.Accounts.Users.__info__(:functions)\n[\n  change_user: 1,\n  change_user: 2,\n  create_user: 0,\n  create_user: 1,\n  create_user!: 0,\n  create_user!: 1,\n  delete_user: 1,\n  delete_user!: 1,\n  get_user: 1,\n  get_user!: 1,\n  get_user_by: 1,\n  get_user_by!: 1,\n  get_user_by: 2,\n  get_user_by!: 2,\n  list_users: 0,\n  list_users: 1,\n  list_users: 2,\n  update_user: 1,\n  update_user: 2,\n  update_user!: 1,\n  update_user!: 2\n]\n```\n\nGenerated creation and updating functions default to the corresponding schema's `changeset/1` and `changeset/2` functions, respectively, whereas list and get functions provide a means to manipulate the result by:\n\n* filtering conditions (via plain exact match condition lists or by passing an Ecto.Query)\n* preloads\n* orderings\n* limits\n* offsets\n\nExamples:\n\n```elixir\n# List all users with posts preloaded\niex> App.Accounts.Users.list_users(preload: [:posts])\n\n# Use an Ecto.Query to filter users, and a keyword list of options to manipulate the result\niex> App.Accounts.Users.list_users(\n  App.Accounts.User |> where([u], u.status == \"active\"),\n  preload: [:posts],\n  order_by: [desc: :inserted_at],\n  limit: 10,\n  offset: 0\n)\n\n# Use a keyword list of exact match conditions and manipulation options\niex> App.Accounts.Users.list_users(\n  status: \"active\",\n  subscription: [plan: \"free\"],\n  order_by: [desc: :inserted_at]\n)\n\n# Get a user by ID with subscription preloaded\niex> App.Accounts.Users.get_user!(10, preload: [:subscription])\n\n# Get a user by profile email with profile and subscription preloaded\niex> App.Accounts.Users.get_user_by!(profile: [email: \"user@example.com\"], preload: [:profile, :subscription])\n\n# Use an Ecto.Query to get a specific user\niex> App.Accounts.Users.get_user_by!(App.Accounts.User |> where([u], u.id == 10), preload: [:profile, :subscription])\n```","ref":"readme.html#usage"},{"type":"extras","title":"Using Ecto Queries - LemonCrud","doc":"LemonCrud provides built-in support for using Ecto queries directly as arguments to the generated functions. This gives you more flexibility and power when you need more complex filtering than what the simple keyword list conditions can provide.","ref":"readme.html#using-ecto-queries"},{"type":"extras","title":"List Functions with Ecto Queries - LemonCrud","doc":"You can pass an Ecto query as the first argument to any `list_*` function:\n\n```elixir\n# Basic query with a WHERE clause\niex> ItemContext.list_items(from(i in Item, where: like(i.name, \"Item 1.2.%\")))\n\n# Combining a query with additional options.\n#\n# Note that the second dargument can still take options as described earlier -\n# they will be appended to the base queryable provided in the first argument.\niex> ItemContext.list_items(\n  from(i in Item, where: like(i.name, \"Item 1.2.%\")),\n  limit: 1,\n  offset: 1,\n  order_by: [desc: :name]\n)\n\n# The schema module can also be used directly, as it's also a queryable.\niex> ItemContext.list_items(Item, limit: 2, offset: 1, order_by: [desc: :name])\n```","ref":"readme.html#list-functions-with-ecto-queries"},{"type":"extras","title":"Get Functions with Ecto Queries - LemonCrud","doc":"Similarly, the `get_*_by` and `get_*_by!` functions can accept Ecto queries - behaving in line with their underlying calls to `Ecto.Repo`'s `get_by` and `get_by!` functions, respectively:\n\n```elixir\n# Find a record with a complex WHERE condition\niex> ItemContext.get_item_by(\n  from(i in Item,\n    where: like(i.name, \"Item 1.1.%\") and i.serial_number == \"1234567890\"\n  )\n)\n\n# Using a query with preloads\niex> CategoryContext.get_category_by(\n  from(c in Category, where: c.id == ^category_id),\n  preload: [:subcategories]\n)\n```\n\nThese query-based approaches are especially useful when:\n\n1. You need complex filtering logic (multiple conditions, OR clauses, etc.)\n2. You want to use SQL functions (like `like`, `in`, etc.)\n3. You need to join with tables that aren't directly accessible via associations\n4. You want to dynamically build queries based on user input\n\nA pattern we use and recommend is keeping contexts clear of bloat from query-building code, which is delegated to specialized query modules - see [our article on query module pattern](https://curiosum.com/blog/composable-elixir-ecto-queries-modules).","ref":"readme.html#get-functions-with-ecto-queries"},{"type":"extras","title":"Query Options - LemonCrud","doc":"When using plain keyword lists for filtering, in addition to `preload`, LemonCrud provides several options for manipulating the constructed Ecto query covering most common use cases: `limit` and `offset` for pagination purposes, and `order_by` for sorting.\n\nAnother option is `count` that joins the base query with subqueries counting related records in a specific association.\n\nIt should be noted that, when more advanced query manipulations are needed, pre-constructed Ecto queries should be used instead of keyword lists in the arguments of the functions.","ref":"readme.html#query-options"},{"type":"extras","title":"limit - LemonCrud","doc":"The `limit` option restricts the number of results returned by the query. Under the hood, it applies `Ecto.Query.limit/2` to your query.\n\n```elixir\n# Get only the first item sorted by name in descending order\niex> ItemContext.list_items(limit: 1, order_by: [desc: :name])\n\n# Apply limit combined with other options\niex> ItemContext.list_items(\n  subcategory_id: subcategory.id,\n  preload: [subcategory: :category],\n  limit: 1,\n  offset: 1,\n  order_by: [desc: :name]\n)\n```","ref":"readme.html#limit"},{"type":"extras","title":"offset - LemonCrud","doc":"The `offset` option skips a specific number of results before returning the rest. This is commonly used with `limit` for pagination. Under the hood, it applies `Ecto.Query.offset/2` to your query.\n\n```elixir\n# Skip the first item and get the next two, sorted by name in descending order\niex> ItemContext.list_items(limit: 2, offset: 1, order_by: [desc: :name])\n\n# Can be used with Ecto.Query as well\niex> ItemContext.list_items(\n  from(i in Item, where: like(i.name, \"Item 1.2.%\")),\n  limit: 1,\n  offset: 1,\n  order_by: [desc: :name]\n)\n```","ref":"readme.html#offset"},{"type":"extras","title":"order_by - LemonCrud","doc":"The `order_by` option sorts the results based on specified fields and directions. Under the hood, it applies `Ecto.Query.order_by/3` to your query.\n\n```elixir\n# Sort items by name in descending order\niex> ItemContext.list_items(order_by: [desc: :name])\n\n# Can be used with filtering conditions\niex> CategoryContext.list_categories(\n  count: [:subcategories, :items],\n  order_by: [desc: :id]\n)\n```","ref":"readme.html#order_by"},{"type":"extras","title":"count - LemonCrud","doc":"The `count` option adds virtual fields to your results with counts of associated records. Under the hood, it adds left-joined subqueries to count the associations and adds the counts as values virtual fields named `[association_name]_count`.\n\nThese virtual fields need to be defined in the schema, preferably defaulting to nil.\n\n```elixir\n# Count associations for all categories\niex> CategoryContext.list_categories(count: [:subcategories, :items])\n# Result example:\n# [\n#   %Category{id: 1, name: \"Category 1\", subcategories_count: 2, items_count: 4},\n#   %Category{id: 2, name: \"Category 2\", subcategories_count: 2, items_count: 4}\n# ]\n\n# Can be combined with other options\niex> CategoryContext.list_categories(\n  id: category.id,\n  count: [:subcategories, :items],\n  order_by: [desc: :id]\n)\n\n# Works with get_by as well\niex> CategoryContext.get_category_by(\n  name: \"Category 1\",\n  count: [:subcategories, :items],\n  preload: [:subcategories]\n)\n```","ref":"readme.html#count"},{"type":"extras","title":"Contributing - LemonCrud","doc":"Pull requests are welcome. For major changes, please open an issue first to discuss what you would like to change.","ref":"readme.html#contributing"},{"type":"extras","title":"Further reading - LemonCrud","doc":"Full documentation can be found at  .","ref":"readme.html#further-reading"},{"type":"extras","title":"License - LemonCrud","doc":"[Curiosum](https://curiosum.com)\n\nDistributed under the MIT License. See [LICENSE](https://github.com/curiosum-dev/lemon_crud/blob/main/LICENSE) for more information.","ref":"readme.html#license"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}